;**********************************************************************************************************************
;	NES Alarm Clock Demo - Notes and design document
;	---------------------------------------------------------------------------------------------------------------
;	2011 Eric Wright (info@thetechnickel.com) thetechnickel.com
;**********************************************************************************************************************

	.inesprg 2												; 32kb PRG ROM
	.ineschr 1												; 8kb CHR ROM
	.inesmap 0												; mapper 0, NROM
	.inesmir 0												; horizontal mirroring
	
;/* Lets define some things! yay! */

; Registers
PPU_CTRL			=	$2000								; PPU Control
PPU_MASK			=	$2001								; PPU Mask
PPU_STATUS			=	$2002								; PPU Status
OAM_ADDR			=	$2003								; OAM Address
OAM_DATA			=	$2004								; OAM Data
PPU_SCROLL			=	$2005								; PPU Scroll
PPU_ADDR			=	$2006								; PPU Address
PPU_DATA			=	$2007								; PPU Data
OAM_DMA				=	$4014								; OAM DMA
SND_CHN				=	$4015								; Sound selection
JOY_1				=	$4016								; Joypad 1
JOY_2				=	$4017								; Joypad 2

; Named values
Max_Seconds			=	$3C									; (60) not really needed, could just put the hex value....
Min_Seconds			=	$00									; (0)  ^-- dito
Max_Hours			=	$0D									; (13) ^-- dito again
Min_Hours			=	$01									; (01) ^-- dito once more! (with feeling this time!)
	
; Zero Page Stuff (Variables)
Temp_A				=	$00									; Temp Accumulator Storage
Temp_X				=	$01									; Temp X Index Storage
Temp_Y				=	$02									; Temp Y Index Storage
NAM_HI				=	$03									; Temp NAM address, High byte
NAM_LO				=	$04									; Temp NAM address, Low byte
PPU_HI				=	$05									; Temp PPU address, High byte
PPU_LO				=	$06									; Temp PPU address, Low byte
PPU_CTRL_Copy		=	$07									; Mirror of PPU_CTRL ($2000)
PPU_MASK_Copy		=	$08									; Mirror of PPU_MASK ($2001)
Frame_Counter		=	$09									; Frame (VBlank) counter (count of 60 = 1 second)
Current_NAM			=	$0A									; the current NAM table we are working with, should always be $20 for the alarm demo

Clock_Hour			=	$20									; Current clock hour
Clock_Minute		=	$21									; Current clock minute
Clock_Second		=	$22									; Current clock second
Clock_Status		=	$23									; Current clock status:
																;	Bit 0 - Clock AM/PM		AM = 0, PM = 1
																;	Bit 1 - Alarm AM/PM		AM = 0, PM = 1
																;	Bit 2 - Alarm Enable	Disabled = 0, Enabled = 1
																;	Bit 3 - Alarm Trigger	Silent = 0, Trigger = 1
Alarm_Hour			=	$24									; What hour is the alarm set for?
Alarm_Minute		=	$25									; What minute is the alarm set for?

Hex2Dec_Tens		=	$26									; Ten's digit of converted number
Hex2Dec_Ones		=	$27									; One's digit of converted number

Joypad_1			=	$28									; Joypad 1 - current frame's joypad values
Joypad_2			=	$29									; Joypad 2 - current frame's joypad values 						(not used in this demo, but still captured)
Joypad_1_Old		=	$2A									; Joypad 1 - last frame's joypad value
Joypad_2_Old		=	$2B									; Joypad 2 - last frame's joypad value 							(not used in this demo, but still captured)
Joypad_1_New		=	$2C									; Joypad 1 - buttons that are newly pressed on current frame
Joypad_2_New		=	$2D									; Joypad 2 - buttons that are newly pressed on current frame	(not used in this demo, but still captured)


	.bank 0
	.org $8000

;______________________________________________________________________________________________________________________
;	Main/Vector routines
;______________________________________________________________________________________________________________________
	
;**********************************************************************************************************************
;	Reset routine - handle power on and reset situations
;**********************************************************************************************************************
Reset:
	sei														; Set interrupts to enable
	cld														; clear decimal mode flag
	ldx #$ff												; X = $FF
	txs														; Stack pointer = X ($FF)
	inx														; X = X + 1 ($FF + 1 = $00)

															; wait for two vblanks. If bit 7 of PPU_STATUS = 1 then we are in vblank
Reset_VBlank_1:												;
	lda PPU_STATUS											; wait for 1st vblank
	bpl Reset_VBlank_1										; if bit 7 of PPU_STATUS, Reset_VBlank_1 and wait for bit 7 to = 0
Reset_VBlank_2:												;
	lda PPU_STATUS											; wait for 2nd vblank
	bpl Reset_VBlank_2										; if bit 7 of PPU_STATUS, Reset_VBlank_2 and wait for bit 7 to = 0
	
	txa														; A = X ($00)
	sta PPU_CTRL											; make sure video is disabled
	sta PPU_MASK											;
	
Reset_Clear_RAM:											; Clear all 7 pages of RAM
	sta $00, X												; Address ($00 + X ) = $00
	sta $0100, X											; Address ($0100 + X ) = $00
	sta $0200, X											; Address ($0200 + X ) = $00
	sta $0300, X											; Address ($0300 + X ) = $00
	sta $0400, X											; Address ($0400 + X ) = $00
	sta $0500, X											; Address ($0500 + X ) = $00
	sta $0600, X											; Address ($0600 + X ) = $00
	sta $0700, X											; Address ($0700 + X ) = $00
	inx														; X = X + 1
	bne Reset_Clear_RAM										; if X != 0, goto Reset_Clear_RAM
	
	lda #$FF												;
Reset_Disable_Sprites:										;
	sta $0200, X											; set all of $0200-$02FF to $FF (255) so that sprites are all off screen
	inx														;
	bne Reset_Disable_Sprites								;
	
	lda #$00												; set OAM address to 0
	sta OAM_ADDR											;
	lda #$02												; tell the OAM_DMA register to load OAM data from $0200 - $02FF
	sta OAM_DMA												;
	
	lda #$0C												; set both the init values of Clock_Hour and Alarm_Hour to $0C (12)
	sta Clock_Hour											;
	sta Alarm_Hour											;
	lda #$00												; set both the init values of Clock_Minute and Clock_Second to $00 (0)
	sta Clock_Minute										;
	sta Clock_Second										;
	lda #Max_Seconds												; set init value of the Frame_Counter to $3C (60)
	sta Frame_Counter										;
	
	jsr PPU_Load_Palette									; load the initial palette

	lda #$20												; set current working NAM table to the $2000 table
	sta Current_NAM											;
	lda #$FF												; and init table to all $FF's (background tile $FF)
	jsr PPU_Clear_NAM										;
	
	jsr Misc_Draw_Background								; draw the clock framing and "2011 thetechnickel.com" text from table
	
	jsr PPU_Set_Plus_32										; set PPU to +32 address increment on read/write
	
	jsr Clock_Redraw_Hour									; Draw the initial clock hour
	jsr Clock_Redraw_Minute									; Draw the initial clock minute
	jsr Clock_Redraw_Status									; Draw the initial clock status (AM/PM and Alarm enable)
	
	ldx #$02												; tell the "PPU_Draw_Digit" to draw in the 3rd place (0 being the 1st, 1 being second, 2 being third, etc)
	lda #$0A												; $0A = ":" 
	jsr PPU_Draw_Digit										; finally draw the ":"
	
	lda #%10000000											; set initial value of PPU_CTRL to NMI enable
	jsr PPU_Save_CTRL										; save PPU_CTRL
	lda #%00011110											; set initial value of PPU_MASK to have sprites and background enabled, with no clipping on left side
	jsr PPU_Save_MASK										; save PPU_MASK
	
Reset_Loop_Of_Doom:											; Main program loop, we are doing everything in NMI so there is nothing here
	jmp Reset_Loop_Of_Doom									;


;**********************************************************************************************************************
;	NMI Routine - VBlank handler
;**********************************************************************************************************************
NMI:

	jsr Misc_Joypad											; grab and process joypad input

	lda Joypad_1_Old										; check the old joypad data for "right"
	and #%00000001											; 	clear all but bit 0
	sta Temp_A												;	save in Temp_A
	lda Joypad_1											; 	load current joypad data
	and #%00000001											;	clear all but bit 0
	cmp Temp_A												; 	compare to Temp_A. if same, redraw Clock digits
	bne No_Redraw_Screen									; 	otherwise jump to No_Redraw_Screen
	
	jsr Clock_Redraw_Hour									; redraw clock hour digits
	jsr Clock_Redraw_Minute									; redraw clock minute digits
	jsr Clock_Redraw_Status									; redraw clock status (AM/PM and Alarm Enable)
No_Redraw_Screen:
	
	lda Joypad_1_New										; load the current "new" button pressed from joypad
	cmp #%00100000											; compare it to "select"
	bne NMI_No_Alarm_Toggle									; if not equal, jump to NMI_No_Alarm_Toggle.
	lda Clock_Status										; otherwise, load the clock status
	eor #%00000100											; toggle the Alarm enable flag
	sta Clock_Status										; save new clock status
	jsr Clock_Redraw_Status									; redraw clock status (clock AM/PM and Alarm Enabled)
NMI_No_Alarm_Toggle:

	; 2 = left, set time
	; 1 = right, set alarm
	lda Joypad_1											; load raw joypad data
	and #%00000011											; clear all but Left and Right flags
	
	cmp #%00000001											; compare to Right 
	beq Alarm_Set_Handler_Jump								; jump to Alarm_Set_Handler if user is holding down "right" (Alarm set)
	
	cmp #%00000010											; compare to Left
	beq Clock_Set_Handler_Jump								; jump to Clock_Set_Handler if user is holding down "Left" (Clock set)

NMI_Handler_Return:



	dec Frame_Counter										; this is the main counter for the NES Alarm clock 
															;	all timing is based off of this counter.
															;	We are going to start at $3C (60) and count down
															;	to avoid extra ops we dont need
															;
	lda Frame_Counter										; load the current value of the Frame_Counter
	bne NMI_Frame_Counter_Continue							; If the counter is > 0, continue on without incrementing
															; 	the minutes
	lda #Max_Seconds												; A = $3C (60)
	sta Frame_Counter										; save A in Frame_Counter
	jsr Clock_Increase_Second								; jump to sub routine to increase the clock minute

	lda #$3F												; set PPU_ADDR to Palette set 0, color 3 location
	sta PPU_ADDR											;
	lda #$02												;
	sta PPU_ADDR											;

	lda Clock_Second										; load clock_second counter
	and #%00000001											; clear all but bit 0
	cmp #$00												; compare to 0
	beq NMI_Set_Second_Color_Gray							; if bit 0 = 0, set palette set 0 color 3 to gray. otherwise set to red

	lda #$15												; write $00 (gray) to palette set 0, color 3
	sta PPU_DATA											;
	jmp NMI_Set_Second_Color_Done							; 
	
NMI_Set_Second_Color_Gray:
	lda #$00												; write $00 (gray) to palette set 0, color 3
	sta PPU_DATA											;
NMI_Set_Second_Color_Done:
	
	lda Clock_Status										; load the clock status in to the accumulator
	and #%00000100											; clear all bits BUT bit 2, the alarm enable bit
	bne NMI_Alarm_Disabled									; if bit 2 is 0, alarm is disabled. jump to NMI_Alarm_Disabled
	
	; alarm is enabled, do things!
	lda Clock_Hour											; load the clock's hour counter
	cmp Alarm_Hour											; compare to the alarm's hour setting
	bne NMI_Alarm_Disabled									; if not equal, jump to NMI_Alarm_Disabled. otherwise continue
	
	lda Clock_Minute										; load the clock's minute counter
	cmp Alarm_Minute										; compare with the alarm's minute setting
	bne NMI_Alarm_Disabled									; if not equal, jump to NMI_Alarm_Disabled. otherwise continue

	; insert alarm code here.
	
NMI_Alarm_Disabled:
NMI_Frame_Counter_Continue:

	lda #$00												; load accumulator with $00, (0)
	sta OAM_ADDR											; save to OAM_ADDR
	sta PPU_SCROLL											; save to PPU_SCROLL
	sta PPU_SCROLL											; save to PPU_SCROLL
	sta PPU_ADDR											; save to PPU_ADDR
	sta PPU_ADDR											; save to PPU_ADDR

;**********************************************************************************************************************
;	Interrupt Routine - no really used for this alarm clock
;**********************************************************************************************************************
IRQ:
	rti

Alarm_Set_Handler_Jump:										; this is a dummy routine that jumps to the real alarm time set handler
	jmp Alarm_Set_Time_Handler								; this was used because branches can only jump 128bytes in either direction

Clock_Set_Handler_Jump:										;; this is a dummy routine that jumps to the real clock time set handler
	jmp Clock_Set_Time_Handler								; this was used because branches can only jump 128bytes in either direction
;______________________________________________________________________________________________________________________
;	Clock Related Sub-Routines
;______________________________________________________________________________________________________________________

;**********************************************************************************************************************
;	Handle user input for setting time
;**********************************************************************************************************************
Clock_Set_Time_Handler:
	lda Joypad_1_New										; load the new button info for joypad 1
	and #%11000000											; and clear all bits but "A" and "B"
	
	cmp #%01000000											; if "A" is pressed and was not pressed before
	beq Clock_Set_Handler_Hour_Increase						; jump to code to increase the hour counter
	
	cmp #%10000000											; if "B" is pressed and was not pressed before
	beq Clock_Set_Handler_Minute_Increase					; jump to code to increase the minute counter
	
	jmp NMI_Handler_Return									; return to the NMI routine to finish clock timer stuff

Clock_Set_Handler_Hour_Increase:
	lda #$00												; write $00 (0) to the seconds counter
	sta Clock_Second										; 
	jsr Clock_Increase_Hour									; increase hour counter with AM/PM toggle on wrap
	jmp NMI_Handler_Return									; return to the NMI routine to finish clock timer stuff

Clock_Set_Handler_Minute_Increase:
	lda #$00												; write $00 (0) to the seconds counter
	sta Clock_Second										; 
	jsr Clock_Increase_Minute								; increase the minute counter with NO hour increase on minute wrap
	jmp NMI_Handler_Return									; return to the NMI routine to finish clock timer stuff
	
;**********************************************************************************************************************
;	Increase the clock's second counter, increase minute on wrap
;**********************************************************************************************************************
Clock_Increase_Second:
	inc Clock_Second										; increase the clock's seconds counter
	lda Clock_Second										; load seconds counter in to the accumulator
	cmp #Max_Seconds												; compare accumulator to $3C (60)
	bne Clock_Increase_Second_Continue						; if not equal, jump to Clock_Increase_Second_Continue

	lda #$00												; load A with $00 (0)
	sta Clock_Second										; save to the clock's seconds counter
	jsr Clock_Increase_Minute_Wrap							; jump to subroutine to increase the clock's minute counter

Clock_Increase_Second_Continue:								; return to where we came from
	rts		

;**********************************************************************************************************************
;	increase clock's minute counter, increase hour on wrap
;**********************************************************************************************************************	
Clock_Increase_Minute_Wrap:
	inc Clock_Minute										; increase the clock's minute counter
	lda Clock_Minute										; load minutes counter in to the accumulator
	cmp #Max_Seconds												; compare accumulator to $3C (60)
	bne Clock_Increase_Minute_Wrap_Continue					; if not equal, jump to Clock_Increase_Minute_Wrap_Continue
	
	lda #$00												; load accumulator with $00 (0)
	sta Clock_Minute										; save to the clock's minutes counter
	
	jsr Clock_Increase_Hour									; jump to subroutine to increase the clock's hour counter

Clock_Increase_Minute_Wrap_Continue:						;
	jsr Clock_Redraw_Minute									; make sure on screen minute is up to date
	rts														; return to where we were called from

;**********************************************************************************************************************
;	increase clock's minute counter, no hour increase on wrap
;**********************************************************************************************************************	
Clock_Increase_Minute:
	inc Clock_Minute										; increase the clock's minute counter
	lda Clock_Minute										; load minutes counter in to the accumulator
	cmp #Max_Seconds												; compare accumulator to $3C (60)
	bne Clock_Increase_Minute_Continue						; if not equal, jump to Clock_Increase_Minute_Wrap_Continue
	
	lda #$00												; load accumulator with $00 (0)
	sta Clock_Minute										; save to the clock's minutes counter

Clock_Increase_Minute_Continue:								;
	jsr Clock_Redraw_Minute									; make sure on screen minute is up to date
	rts														; return to where we were called from
	
;**********************************************************************************************************************
;	increase clock's hour counter, toggele AM/PM on wrap
;**********************************************************************************************************************
Clock_Increase_Hour:
	inc Clock_Hour											; increase the clock's hour counter
	lda Clock_Hour											; load the hour counter in to the accumulator

	cmp #$0C												; check if the hour counter is $0C (12)
	bne Clock_Increase_Hour_Dont_Toggle_AMPM				; dont toggel AM/PM if not
	jsr Clock_AMPM_Toggle									; jump to subroutine to toggle the AM/PM flag
	
Clock_Increase_Hour_Dont_Toggle_AMPM:						;
	cmp #Max_Hours												; compare accumulator to $0D (13)
	bne Clock_Increase_Hour_Continue						; if not equal, jump to HourIncrease_Continue
	
	lda #$01												; load accumulator with $01 (1)
	sta Clock_Hour											; save to the clock's hour counter
	
Clock_Increase_Hour_Continue:								;
	jsr Clock_Redraw_Hour									; make sure the on screen hour is up to date
	rts														; return to where we were called from

;**********************************************************************************************************************
;	Toggle the clock's AM/PM flag
;**********************************************************************************************************************
Clock_AMPM_Toggle:
	lda Clock_Status										; load the clock status byte in to the accumulator
	eor #%00000001											; perform an Exclusive OR against bit 0 ( 0 = 1, 1 = 0)
	sta Clock_Status										; save the new clock status bit
	
	jsr Clock_Redraw_Status									; make sure the clock status is up to date
	rts														; return to where we were called from
	
;**********************************************************************************************************************
;	Draw the hour digits on the screen
;**********************************************************************************************************************	
Clock_Redraw_Hour:
	lda Clock_Hour											; load the hour counter
	jsr Misc_Hex2Dec										; send counter to Misc_Hex2Dec function
	
	ldx #$00												; tell PPU_Draw_Digit we need to work in the 1st column of the clock display
	lda Hex2Dec_Tens										; and to use the Ten's digit returned by Misc_Hex2Dec
	jsr PPU_Draw_Digit										; re-draw the digit on screen
	
	ldx #$01												; tell PPU_Draw_Digit we need to work in the 2nd column of the clock display
	lda Hex2Dec_Ones										; and to use the One's digit returned by Misc_Hex2Dec
	jsr PPU_Draw_Digit										; re-draw the digit on screen
	rts														;

;**********************************************************************************************************************
;	Draw the minute digits on the screen
;**********************************************************************************************************************	
Clock_Redraw_Minute:
	lda Clock_Minute										; load the minute counter
	jsr Misc_Hex2Dec										; send counter to the Misc_Hex2Dec function
	
	ldx #$03												; tell PPU_Draw_Digit we need to work in the 4th column of the clock display
	lda Hex2Dec_Tens										; and to use the Ten's digit returned by Misc_Hex2Dec
	jsr PPU_Draw_Digit										; re-draw the digit on screen
	
	ldx #$04												; tell PPU_Draw_Digit we need to work in the 5th column of the clock display
	lda Hex2Dec_Ones										; and to use the One's digit returned by Misc_Hex2Dec
	jsr PPU_Draw_Digit										; re-draw the digit on screen
	rts														;
	
;**********************************************************************************************************************
;	Draw the clock status on the screen (AM/PM and Alarm Enabled)
;**********************************************************************************************************************
Clock_Redraw_Status:
	lda #$21												; set PPU_ADDR for AM/PM flag location on NAM
	sta PPU_ADDR											;
	lda #$D2												;
	sta PPU_ADDR											;
	
	lda Clock_Status										; load the clock status
	pha 													; save copy of status to stack
	and #%00000001											; clear all bits but AM/PM bit
	beq Clock_Redraw_Status_AM								; if AM/PM not set, jump to code ot draw blank tile for "AM"
	lda #$66												; otherwise load tile $66 "PM DOT"
	sta PPU_DATA											; save to PPU_DATA
	jmp Redraw_Status_AMPM_Done								; jump to the finish of the AMPM flag routine
Clock_Redraw_Status_AM:										;
	lda #$76												; load the blank tile for "AM"
	sta PPU_DATA											; save to PPU_DATA
Redraw_Status_AMPM_Done:									;

	lda #$22												; set PPU_ADDR for Alarm enable flag on NAM
	sta PPU_ADDR											;
	lda #$12												;
	sta PPU_ADDR											;

	pla														; pull clock status from stack
	and #%00000100											; clear all bits but Alarm Enable bit
	
	beq Clock_Redraw_Status_Alarm_Disable					; if alarm not set, jump to code to draw blank title for "alarm disabled"
	lda #$86												; otherwise load tile $86 (alarm enabled dot)
	sta PPU_DATA											; save to PPU_DATA
	jmp Clock_Redraw_Status_Alarm_Done						; jump to the finish of the alarm enable routine
Clock_Redraw_Status_Alarm_Disable:							;
	lda #$76												; lod the blank tile for "alarm disabled"
	sta PPU_DATA											; save to PPU_DATA
Clock_Redraw_Status_Alarm_Done:								;
	rts														;

	
;______________________________________________________________________________________________________________________
;	Alarm Related Sub-Routines
;______________________________________________________________________________________________________________________

;**********************************************************************************************************************
;	Handle user input for setting time
;**********************************************************************************************************************
Alarm_Set_Time_Handler:
	lda Joypad_1_New
	and #%00000001
	bne Alarm_Set_Time_Handler_Dont_Redraw

	jsr Alarm_Redraw_Hour
	jsr Alarm_Redraw_Minute
	jsr Alarm_Redraw_Status
	
Alarm_Set_Time_Handler_Dont_Redraw:

	lda Joypad_1_New										; load the new button info for joypad 1
	and #%11000000											; and clear all bits but "A" and "B"
	
	cmp #%01000000											; if "A" is pressed and was not pressed before
	beq Alarm_Set_Handler_Hour_Increase						; jump to code to increase the hour counter
	
	cmp #%10000000											; if "B" is pressed and was not pressed before
	beq Alarm_Set_Handler_Minute_Increase					; jump to code to increase the minute counter
	
	jmp NMI_Handler_Return									; return to the NMI routine to finish clock timer stuff

Alarm_Set_Handler_Hour_Increase:
	jsr Alarm_Increase_Hour									; increase hour counter with AM/PM toggle on wrap
	jmp NMI_Handler_Return									; return to the NMI routine to finish clock timer stuff

Alarm_Set_Handler_Minute_Increase:
	jsr Alarm_Increase_Minute								; increase the minute counter with NO hour increase on minute wrap
	jmp NMI_Handler_Return									; return to the NMI routine to finish clock timer stuff

;**********************************************************************************************************************
;	increase Alarm's minute counter, no hour increase on wrap
;**********************************************************************************************************************	
Alarm_Increase_Minute:
	inc Alarm_Minute										; increase the Alarm's minute counter
	lda Alarm_Minute										; load minutes counter in to the accumulator
	cmp #Max_Seconds												; compare accumulator to $3C (60)
	bne Alarm_Increase_Minute_Continue						; if not equal, jump to Alarm_Increase_Minute_Wrap_Continue
	
	lda #$00												; load accumulator with $00 (0)
	sta Alarm_Minute										; save to the Alarm's minutes counter

Alarm_Increase_Minute_Continue:								;
	jsr Alarm_Redraw_Minute									; make sure the on screen minute is up to date
	rts														; return to where we were called from
	
;**********************************************************************************************************************
;	increase Alarm's hour counter, toggele AM/PM on wrap
;**********************************************************************************************************************
Alarm_Increase_Hour:
	inc Alarm_Hour											; increase the Alarm's hour counter
	lda Alarm_Hour											; load the hour counter in to the accumulator

	cmp #$0C												;
	bne Alarm_Increase_Hour_Dont_Toggle_AMPM				;
	jsr Alarm_AMPM_Toggle									; jump to subroutine to toggle the AM/PM flag
	
Alarm_Increase_Hour_Dont_Toggle_AMPM:						;
	cmp #Max_Hours												; compare accumulator to $0D (13)
	bne Alarm_Increase_Hour_Continue						; if not equal, jump to HourIncrease_Continue
	
	lda #$01												; load accumulator with $01 (1)
	sta Alarm_Hour											; save to the Alarm's hour counter
	
Alarm_Increase_Hour_Continue:								;
	jsr Alarm_Redraw_Hour									; make sure the on screen hour is up to date
	rts														; return to where we were called from

;**********************************************************************************************************************
;	Toggle the Alarm's AM/PM flag
;**********************************************************************************************************************
Alarm_AMPM_Toggle:
	lda Clock_Status										; load the Alarm status byte in to the accumulator
	eor #%00000010											; perform an Exclusive OR against bit 0 ( 0 = 1, 1 = 0)
	sta Clock_Status										; save the new Alarm status bit
	
	jsr Alarm_Redraw_Status									; make sure the on screen status is up to date (alarm AM/PM and Alarm Enabled)
	rts														; return to where we were called from
	
;**********************************************************************************************************************
;	Draw the hour digits on the screen
;**********************************************************************************************************************	
Alarm_Redraw_Hour:
	lda Alarm_Hour											; load the hour counter
	jsr Misc_Hex2Dec										; send counter to Misc_Hex2Dec function
	
	ldx #$00												; tell PPU_Draw_Digit we need to work in the 1st column of the clock display
	lda Hex2Dec_Tens										; and to use the Ten's digit returned by Misc_Hex2Dec
	jsr PPU_Draw_Digit										; re-draw the digit on screen
	
	ldx #$01												; tell PPU_Draw_Digit we need to work in the 2nd column of the clock display
	lda Hex2Dec_Ones										; and to use the One's digit returned by Misc_Hex2Dec
	jsr PPU_Draw_Digit										; re-draw the digit on screen
	rts														;

;**********************************************************************************************************************
;	Draw the minute digits on the screen
;**********************************************************************************************************************	
Alarm_Redraw_Minute:
	lda Alarm_Minute										; load the minute counter
	jsr Misc_Hex2Dec										; send counter to the Misc_Hex2Dec function
	
	ldx #$03												; tell PPU_Draw_Digit we need to work in the 4th column of the clock display
	lda Hex2Dec_Tens										; and to use the Ten's digit returned by Misc_Hex2Dec
	jsr PPU_Draw_Digit										; re-draw the digit on screen
	
	ldx #$04												; tell PPU_Draw_Digit we need to work in the 5th column of the clock display
	lda Hex2Dec_Ones										; and to use the One's digit returned by Misc_Hex2Dec
	jsr PPU_Draw_Digit										; re-draw the digit on screen
	rts		
	
;**********************************************************************************************************************
;	Draw the Alarm status on the screen (AM/PM and Alarm Enabled)
;**********************************************************************************************************************
Alarm_Redraw_Status:
	lda #$21												; set PPU_ADDR for AM/PM flag location on NAM
	sta PPU_ADDR											;
	lda #$D2												;
	sta PPU_ADDR											;
	
	lda Clock_Status										; load the clock status
	pha 													; save copy of status to stack
	and #%00000010											; clear all bits but Alarm AM/PM bit
	beq Alarm_Redraw_Status_AM								; if AM/PM not set, jump to code ot draw blank tile for "AM"
	lda #$66												; otherwise load tile $66 "PM DOT"
	sta PPU_DATA											; save to PPU_DATA
	jmp Alarm_Redraw_Status_AMPM_Done								; jump to the finish of the AMPM flag routine
Alarm_Redraw_Status_AM:										;
	lda #$76												; load the blank tile for "AM"
	sta PPU_DATA											; save to PPU_DATA
Alarm_Redraw_Status_AMPM_Done:								;

	lda #$22												; set PPU_ADDR for Alarm enable flag on NAM
	sta PPU_ADDR											;
	lda #$12												;
	sta PPU_ADDR											;

	pla														; pull clock status from stack
	and #%000000100											; clear all bits but Alarm Enable bit
	
	beq Alarm_Redraw_Status_Alarm_Disable					; if alarm not set, jump to code to draw blank title for "alarm disabled"
	lda #$86												; otherwise load tile $86 (alarm enabled dot)
	sta PPU_DATA											; save to PPU_DATA
	jmp Alarm_Redraw_Status_Alarm_Done						; jump to the finish of the alarm enable routine
Alarm_Redraw_Status_Alarm_Disable:							;
	lda #$76												; lod the blank tile for "alarm disabled"
	sta PPU_DATA											; save to PPU_DATA
Alarm_Redraw_Status_Alarm_Done:								;
	rts														;

	
;______________________________________________________________________________________________________________________
;	PPU Related Sub-Routines
;______________________________________________________________________________________________________________________

;**********************************************************************************************************************
;	Load the palette from palette table
;**********************************************************************************************************************	
PPU_Load_Palette:
	lda #$3F												; set PPU_ADDR to palette ram, $3F00
	sta PPU_ADDR											;
	lda #$00												;
	sta PPU_ADDR											;
	tax														; set X to 0

PPU_Load_Palette_2:
	lda PPU_Load_Palette_Data, x							; load palette data from PPU_Load_Palette_Data + x
	sta PPU_DATA											; save to PPU_DATA
	inx														; increase X
	cpx #$20												; does X = $20 (32)
	bne PPU_Load_Palette_2									; if not, jump to PPU_Load_Palette_2
	rts														; 

PPU_Load_Palette_Data:
	; nam palette data
	.db $0F, $15, $00, $10									; set 0
	.db $0F, $15, $00, $10									; set 1
	.db $0F, $15, $00, $10									; set 2
	.db $0F, $15, $00, $10									; set 3
	
	; oam palette data
	.db $0F, $00, $00, $00									; set 0
	.db $0F, $00, $00, $00									; set 1
	.db $0F, $00, $00, $00									; set 2
	.db $0F, $00, $00, $00									; set 3
	

;**********************************************************************************************************************
;	Clear the current NAM table with value in Accumulator
;**********************************************************************************************************************
PPU_Clear_NAM:
	pha														; save Accumulator tile to stack (temp storage)
	
	jsr PPU_Set_Plus_1										; make sure we are using +1 address increment
	
	lda Current_NAM											; load the high-byte of the NAM table we are working with
	sta PPU_ADDR											; save it to the PPU address register
	lda #$00												; load $00 for the low-byte of the NAM table address
	sta PPU_ADDR											; save it to the PPU address register

	pla														; pull the tile off the stack and save back to the accumulator
	ldx #$F0												; load X with 240 (240 * 4 = 960 bytes, the size of the NAM table)
	ldy #$04												; load Y with 4
PPU_Clear_NAM_2:												
	sta PPU_DATA											; save the tile to the PPU data register
	dex														; decrease X by 1
	bne PPU_Clear_NAM_2										; if X != 0, goto PPU_Clear_NAM_2
	ldx #$F0												; load X back to 240
	dey														; dey
	bne PPU_Clear_NAM_2										; if Y != 0, go to PPU_Clear_NAM_2
	rts		
	
;**********************************************************************************************************************
;	Set CTRL bits and save to CTRL Copy in RAM and PPU_CTRL register
;**********************************************************************************************************************
PPU_Save_CTRL:
	sta PPU_CTRL_Copy										; save value of A in to PPU_CTRL_Copy
	sta PPU_CTRL											; save value of A in to PPU_CTRL
	rts
	
;**********************************************************************************************************************
;	Set MASK bits and save to MASK Copy in RAM and PPU_MASK register
;**********************************************************************************************************************
PPU_Save_MASK:
	sta PPU_MASK_Copy										; save value of A in to PPU_MASK_Copy
	sta PPU_MASK											; save value of A in to PPU_MASK
	rts

;**********************************************************************************************************************
;	Set PPU to +1 address increment on PPU_DATA write
;**********************************************************************************************************************
PPU_Set_Plus_1:
	lda PPU_CTRL_Copy										; load PPU_CTRL_Copy
	and #%11111011											; clear the address increment bit, setting mode to +1 increment
	sta PPU_CTRL_Copy										; save to PPU_CTRL_Copy
	sta PPU_CTRL											; save to PPU_CTRL
	rts

;**********************************************************************************************************************
;	Set PPU to +32 address increment on PPU_DATA write
;**********************************************************************************************************************
PPU_Set_Plus_32:
	lda PPU_CTRL_Copy										; load PPU_CTRL_Copy
	ora #%00000100											; set the address increment bit, setting mode to +32 increment
	sta PPU_CTRL_Copy										; save to PPU_CTRL_Copy
	sta PPU_CTRL											; save to PPU_CTRL
	rts

;**********************************************************************************************************************
;	Draw a "ditial digit" on the screen in one of 5 locations
;**********************************************************************************************************************
PPU_Draw_Digit:
		
	pha														; save the digit we will be working with to the stack

	jsr PPU_Set_Plus_32										; set PPU mode to +32 address increment
	
	lda #$21												; save the high-byte of the NAM address in to NAM_HI
	sta NAM_HI												;
	lda PPU_Draw_Digit_Address_Table, x						; load the low byte from PPU_Draw_Digit_Address_Table + X (where X is the digit column)
	sta NAM_LO												; save the low-byte of the NAM address in to NAM_LO
	
	pla														; pull the digit back off the stack
	asl a													; shift bits left by one (A = A * 2)
	asl a													; shift bits left by one (A = A * 2)
	asl a													; shift bits left by one (A = A * 2)
	tax														; transfer A to X to be used as an index
	ldy #$02												; set Y to $02 (2)
PPU_Draw_Digit_2:											
	lda NAM_HI												; load NAM address from NAM_HI and NAM_LO and save to PPU_ADDR
	sta PPU_ADDR											;
	lda NAM_LO												;
	sta PPU_ADDR											;

	lda PPU_Draw_Digit_Tile_Table, x						; Load top tile for digit from PPU_Draw_Digit_Tile_Table + x
	sta PPU_DATA											; save to PPU_DATA
	inx 													; increase X index
	
	lda PPU_Draw_Digit_Tile_Table, x						; Load top tile for digit from PPU_Draw_Digit_Tile_Table + x
	sta PPU_DATA											; save to PPU_DATA
	inx 													; increase X index
	
	lda PPU_Draw_Digit_Tile_Table, x						; Load top tile for digit from PPU_Draw_Digit_Tile_Table + x
	sta PPU_DATA											; save to PPU_DATA
	inx 													; increase X index
	
	inc NAM_LO												; increase the NAM address by 1
	dey														; decrease Y (we need to run the loop twice)
	bne PPU_Draw_Digit_2									; if Y > 0, jump to PPU_Draw_Digit_2
	rts														;
	
	
PPU_Draw_Digit_Address_Table:
	.db $C7, $C9, $CB, $CD, $CF								; Initial NAM_LO values for each of the 5 digit columns

PPU_Draw_Digit_Tile_Table:
	.db $62,$72,$82,$63,$73,$83,$00,$00						; tiles for "0"
	.db $30,$40,$50,$31,$41,$51,$00,$00						; tiles for "1"
	.db $32,$42,$52,$33,$43,$53,$00,$00						; tiles for "2"
	.db $34,$44,$54,$35,$45,$55,$00,$00						; tiles for "3"
	.db $36,$46,$56,$37,$47,$57,$00,$00						; tiles for "4"
	.db $38,$48,$58,$39,$49,$59,$00,$00						; tiles for "5"
	.db $3A,$4A,$5A,$3B,$4B,$5B,$00,$00						; tiles for "6"
	.db $3C,$4C,$5C,$3D,$4D,$5D,$00,$00						; tiles for "7"
	.db $3E,$4E,$5E,$3F,$4F,$5F,$00,$00						; tiles for "8"
	.db $60,$70,$80,$61,$71,$81,$00,$00						; tiles for "9"
	.db $64,$74,$84,$65,$75,$85,$00,$00						; tiles for ":"

	
;______________________________________________________________________________________________________________________
;	Misc / Other Sub-Routines
;______________________________________________________________________________________________________________________

;**********************************************************************************************************************
;	Strobe joypad latches and grab current button values from both Joypad registers (JOY_1 & JOY_2)
;**********************************************************************************************************************
Misc_Joypad:
	ldx #$01												; strobe the joypads
	stx JOY_1												; write 1 to JOY_1
	dex														;
	stx JOY_1												; write 0 to JOY_1

Misc_Joypad_2:
	lda Joypad_1, x											; save old values for joypad "X"
	eor #$FF												; invert the previous results
	sta Joypad_1_Old, x										; save as old joypad data

	ldy #$08												; we need to do this 8 times to grab all 8 buttons
Misc_Joypad_3:												;
	ror JOY_1, x											; roll bit 0 onto carry flag
	rol a													; roll carry flag on to A

	dey														;
	bne Misc_Joypad_3										; loop if we have not loaded 8 buttons yet

	sta Joypad_1, x											; save the joypad input
	and Joypad_1_Old, x										; and against the inverted previous results
	sta Joypad_1_New, x										; save for "new" button hits
	
	inx														; loop if we have not loaded both joypads
	cpx #$02												;
	bne Misc_Joypad_2										;
	rts														; go back to where we came from

;**********************************************************************************************************************
;	Convert HEX number to DEC using lookup table, storing results in Hex2Dec_Tens and Hex2Dec_Ones
;**********************************************************************************************************************
Misc_Hex2Dec:
	tax														; transfer A to X to be used as index
	lda Misc_Hex2Dec_Table, x								; load Dec value from Misc_Hex2Dec_Table
	pha														; save copy to stack
	and #%00001111											; clear all but the lower 4 bits
	sta Hex2Dec_Ones										; save as the One's result
	
	pla														; pull saved value from stack
	lsr	a													; shift bits right by 1 (A = A / 2)
	lsr a													; shift bits right by 1 (A = A / 2)
	lsr a													; shift bits right by 1 (A = A / 2)
	lsr a													; shift bits right by 1 (A = A / 2)
	and #%00001111											; clear all but the lower 4 bits
	sta Hex2Dec_Tens										; save as the Ten's result
	rts														;

Misc_Hex2Dec_Table:
	.db $00, $01, $02, $03, $04, $05, $06, $07, $08, $09	
	.db $10, $11, $12, $13, $14, $15, $16, $17, $18, $19	
	.db $20, $21, $22, $23, $24, $25, $26, $27, $28, $29	
	.db $30, $31, $32, $33, $34, $35, $36, $37, $38, $39	
	.db $40, $41, $42, $43, $44, $45, $46, $47, $48, $49	
	.db $50, $51, $52, $53, $54, $55, $56, $57, $58, $59	
	

;**********************************************************************************************************************
;	Draw the clock BG and labels
;**********************************************************************************************************************
Misc_Draw_Background:
	ldx #$00												; make sure the starting X index is $00 (0)

Misc_Draw_Background_2:
	lda Misc_Draw_Background_Table, x						; load the NAM Hi-Byte address from Misc_Draw_Background_Table + X
	sta PPU_ADDR											; save to PPU_ADDR
	inx														; increase X index
	
	lda Misc_Draw_Background_Table, x						; load the NAM Lo-Byte address from Misc_Draw_Background_Table + X
	sta PPU_ADDR											; save to PPU_ADDR
	inx														; increase X index
	
Misc_Draw_Background_3:
	lda Misc_Draw_Background_Table, x						; load data from Misc_Draw_Background_Table + X
	inx														; increase X index
	cmp #$FE												; check to see if A = $FE (triggering new NAM address)
	beq Misc_Draw_Background_2								; jump to Misc_Draw_Background_2 if A = $FE
	
	cmp #$FF												; check to see if A = $FF (Done drawing screen)
	beq Misc_Draw_Background_4								; jump to Misc_Draw_Background_4 if A = $FF

	sta PPU_DATA											; save data to PPU_DATA
	jmp Misc_Draw_Background_3								; jump to Misc_Draw_Background_3 to continue drawing loop

Misc_Draw_Background_4:
	rts
	
Misc_Draw_Background_Table:
	.db $21, $A6, $67, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $69, $FE
	.db $21, $C6, $6A, $30, $30, $30, $30, $30, $30, $30, $30, $30, $30, $30, $76, $6F, $19, $16, $6F, $6F, $6F, $6F, $6B, $FE
	.db $21, $E6, $6A, $30, $30, $30, $30, $30, $30, $30, $30, $30, $30, $30, $76, $6F, $6F, $6F, $6F, $6F, $6F, $6F, $6B, $FE
	.db $22, $06, $6A, $30, $30, $30, $30, $30, $30, $30, $30, $30, $30, $30, $76, $6F, $0A, $15, $0A, $1B, $16, $6F, $6B, $FE
	.db $22, $26, $6C, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6E, $FE
	.db $23, $20
	.db $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68
	.db $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68, $68
	.db $6F, $6F, $6F, $6F, $6F, $02, $00, $01, $01, $6F, $6F, $1D, $11, $0E, $1D, $0E
	.db $0C, $11, $17, $12, $0C, $14, $0E, $15, $24, $0C, $18, $16, $6F, $6F, $6F, $6F
	.db $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D
	.db $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D, $6D
	.db $FF
	
;______________________________________________________________________________________________________________________
;	Vectors and such
;______________________________________________________________________________________________________________________

	.bank 1
	.bank 2
	.bank 3
	
	; setup our vector table
	.org $FFFA
	.dw NMI													; Address to our NMI routine
	.dw Reset												; Address to our Startup / Reset routine
	.dw	IRQ													; Address to our IRQ routine
	
	.org $0000
	.bank 4
	.incbin "nesalarm.chr"
